package com.huaxia.hx_sell_commission.utils;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * 金额计算工具类
 * @Author shilei
 * Date 2020/12/23
 */
public class MoneyUtils {

    // 默认运算精度
    private static final Integer DEF_DIV_SCALE = 0;

    /**
     * 加法运算。
     *
     * @param value1 被加数
     * @param value2 加数
     * @return 两个参数的和
     */
    public static BigDecimal add(String value1, String value2) {
        BigDecimal b1 = new BigDecimal(value1);
        BigDecimal b2 = new BigDecimal(value2);
        return b1.add(b2);
    }

    /**
     * 减法运算。
     *
     * @param value1 被减数
     * @param value2 减数
     * @return 两个参数的差
     */
    public static BigDecimal subtract(String value1, String value2) {
        BigDecimal b1 = new BigDecimal(value1);
        BigDecimal b2 = new BigDecimal(value2);
        return b1.subtract(b2);
    }

    /**
     * 乘法运算，向下取整。
     *
     * @param value1 被乘数
     * @param value2 乘数
     * @return 两个参数的积
     */
    public static BigDecimal multiply(String value1, String value2) {
        BigDecimal b1 = new BigDecimal(value1);
        BigDecimal b2 = new BigDecimal(value2);
        return b1.multiply(b2).setScale(DEF_DIV_SCALE, BigDecimal.ROUND_DOWN);
    }

    /**
     * 乘法运算，向下取整。
     *
     * @param value1 被乘数
     * @param value2 乘数
     * @param value3 乘数
     * @return 两个参数的积
     */
    public static BigDecimal multiply(String value1, String value2, String value3) {
        BigDecimal b1 = new BigDecimal(value1);
        BigDecimal b2 = new BigDecimal(value2);
        BigDecimal b3 = new BigDecimal(value3);
        return b1.multiply(b2).multiply(b3).setScale(DEF_DIV_SCALE, BigDecimal.ROUND_DOWN);
    }

    /**
     * 乘法运算，四舍五入。
     *
     * @param value1 被乘数
     * @param value2 乘数
     * @param scale 表示需要精确到小数点后几位。
     * @return 两个参数的积
     */
    public static BigDecimal multiplyHalfUp(String value1, String value2, Integer scale) {
        if (scale < 0) {
            throw new IllegalArgumentException("The scale must be a positive integer or zero");
        }
        BigDecimal b1 = new BigDecimal(value1);
        BigDecimal b2 = new BigDecimal(value2);
        return b1.multiply(b2).setScale(scale, RoundingMode.HALF_UP);
    }

    /**
     * 乘法运算，银行家舍入法。
     *
     * @param value1 被乘数
     * @param value2 乘数
     * @param value3 乘数
     * @param scale 表示需要精确到小数点后几位。
     * @return 两个参数的积
     */
    public static BigDecimal multiplyHalfEven(String value1, String value2,String value3, Integer scale) {
        BigDecimal b1 = new BigDecimal(value1);
        BigDecimal b2 = new BigDecimal(value2);
        BigDecimal b3 = new BigDecimal(value3);
        return b1.multiply(b2).multiply(b3).setScale(scale, RoundingMode.HALF_EVEN);
    }

    /**
     * 除法运算，向下取整。
     *
     * @param dividend 被除数
     * @param divisor  除数
     * @return 两个参数的商
     */
    public static BigDecimal divide(String dividend, String divisor) {
        BigDecimal b1 = new BigDecimal(dividend);
        BigDecimal b2 = new BigDecimal(divisor);
        return b1.divide(b2, DEF_DIV_SCALE, BigDecimal.ROUND_DOWN);
    }

    /**
     * 除法运算，四舍五入。
     *
     * @param dividend 被除数
     * @param divisor 除数
     * @param scale 表示需要精确到小数点以后几位。
     * @return 两个参数的商
     */
    public static BigDecimal divideHalfUp(String dividend, String divisor, Integer scale) {
        if (scale < 0) {
            throw new IllegalArgumentException("The scale must be a positive integer or zero");
        }
        BigDecimal b1 = new BigDecimal(dividend);
        BigDecimal b2 = new BigDecimal(divisor);
        return b1.divide(b2, scale, RoundingMode.HALF_UP);
    }

    /**
     * 除法运算，银行家舍入法。
     *
     * @param dividend 被除数
     * @param divisor 除数
     * @param scale 表示需要精确到小数点以后几位。
     * @return 两个参数的商
     */
    public static BigDecimal divideHalfEven(String dividend, String divisor, Integer scale) {
        if (scale < 0) {
            throw new IllegalArgumentException("The scale must be a positive integer or zero");
        }
        BigDecimal b1 = new BigDecimal(dividend);
        BigDecimal b2 = new BigDecimal(divisor);
        return b1.divide(b2, scale, RoundingMode.HALF_EVEN);
    }

    /**
     * 提供指定数值的（精确）小数位四舍五入处理。
     *
     * @param value 需要四舍五入的数字
     * @param scale 小数点后保留几位
     * @return 四舍五入后的结果
     */
    public static BigDecimal round(String value,int scale){
        if(scale<0){
            throw new IllegalArgumentException("The scale must be a positive integer or zero");
        }
        BigDecimal b = new BigDecimal(value);
        BigDecimal one = new BigDecimal("1");
        return b.setScale(scale, RoundingMode.HALF_UP);
    }

}
